@@ -73,7 +73,9 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
             pub use checkout_core::dropshot::HttpServerStarter;
             pub use checkout_core::error::Error;
             pub use checkout_core::server::Server;
+            pub use checkout_core::internal_context::InternalContext;
 
+            use std::env;
             use std::sync::Arc;
             use checkout_core::dropshot;
             use checkout_core::dropshot::endpoint;
@@ -84,26 +86,39 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
             use checkout_core::dropshot::HttpResponseOk;
             use checkout_core::http::StatusCode;
             use checkout_core::checkout::Checkout;
+            use checkout_core::payment::Payment;
+            use checkout_core::payment::PaymentProcessor;
             use checkout_core::customer::Contact;
             use checkout_core::customer::Address;
             use checkout_core::shipping::FulfillmentTypeSelection;
-            use checkout_core::server::StandardPathParams;
+            use checkout_core::server::IdParam;
             use checkout_core::server::CreateBodyParams;
             use checkout_core::server::AddOrRemoveItemBodyParams;
-            use checkout_core::store::CheckoutStore;
             use checkout_core::transaction::TransactionController;
             use checkout_core::context::Context;
             use checkout_core::context::ContextProvider;
+            use checkout_core::checkout::handlers::{create_handler, add_item_handler, remove_item_handler, confirm_items_handler, update_contact_handler, update_shipping_address_handler, update_fulfillment_type_handler, initiate_payment_handler};
+
+            pub struct Config {
+                google_application_credentials: String
+            }
+
+            pub fn get_config() -> Config {
+                ServerConfig {
+                    google_application_credentials: env::var("GOOGLE_APPLICATION_CREDENTIALS").unwrap(),
+                }
+            }
 
             pub struct ServerContext {
-                pub context_provider: super::#server_type_name
+                pub internal_context: InternalContext,
+                pub context_provider: super::#server_type_name,
             }
 
             #[endpoint {
                 method = POST,
                 path = "/checkout",
             }]
-            pub async fn create_handler(
+            pub async fn create_endpoint(
                 req_ctx: Arc<RequestContext<ServerContext>>,
                 body: TypedBody<CreateBodyParams>,
             ) -> Result<HttpResponseOk<Checkout>, HttpError> {
@@ -116,9 +131,8 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
                     .unwrap();
 
                 ctx.start_transaction().await.unwrap();
-                match Checkout::create(&mut ctx, params.currency.clone(), params.contact.clone(), params.shipping_address.clone()).await {
+                match create_handler(&mut ctx, params.currency.clone(), params.contact.clone(), params.shipping_address.clone()).await {
                     Ok(co) => {
-                        ctx.set_checkout(&co.id, &co).await.unwrap();
                         ctx.commit_transaction().await.unwrap();
                         Ok(HttpResponseOk(co))
                     }
@@ -138,9 +152,9 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
                 method = PATCH,
                 path = "/checkout/{id}/add_item",
             }]
-            pub async fn add_item_handler(
+            pub async fn add_item_endpoint(
                 req_ctx: Arc<RequestContext<ServerContext>>,
-                path: Path<StandardPathParams>,
+                path: Path<IdParam>,
                 body: TypedBody<AddOrRemoveItemBodyParams>,
             ) -> Result<HttpResponseOk<Checkout>, HttpError> {
                 let path_params = path.into_inner();
@@ -153,36 +167,8 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
                     .unwrap();
 
                 ctx.start_transaction().await.unwrap();
-
-                let result = ctx.get_checkout(&path_params.id).await;
-
-                if let Err(err) = result {
-                    ctx.abort_transaction().await.unwrap();
-                    return Err(HttpError {
-                        status_code: StatusCode::BAD_REQUEST,
-                        error_code: Some(err.code.clone()),
-                        external_message: err.message.clone(),
-                        internal_message: err.message.clone(),
-                    });
-                }
-
-                let maybe_co = result.unwrap();
-
-                if maybe_co.is_none() {
-                    ctx.abort_transaction().await.unwrap();
-                    return Err(HttpError {
-                        status_code: StatusCode::NOT_FOUND,
-                        error_code: Some("NOT_FOUND".to_string()),
-                        external_message: "the requested checkout could not be found".to_string(),
-                        internal_message: "the requested checkout could not be found".to_string(),
-                    });
-                }
-
-                let mut co = maybe_co.unwrap();
-
-                match co.add_item(&mut ctx, body_params.sku.clone(), body_params.quantity).await {
-                    Ok(_) => {
-                        ctx.set_checkout(&co.id, &co).await.unwrap();
+                match add_item_handler(&mut ctx, &path_params.id, body_params.sku.clone(), body_params.quantity).await {
+                    Ok(co) => {
                         ctx.commit_transaction().await.unwrap();
                         Ok(HttpResponseOk(co))
                     }
@@ -202,9 +188,9 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
                 method = PATCH,
                 path = "/checkout/{id}/remove_item",
             }]
-            pub async fn remove_item_handler(
+            pub async fn remove_item_endpoint(
                 req_ctx: Arc<RequestContext<ServerContext>>,
-                path: Path<StandardPathParams>,
+                path: Path<IdParam>,
                 body: TypedBody<AddOrRemoveItemBodyParams>,
             ) -> Result<HttpResponseOk<Checkout>, HttpError> {
                 let path_params = path.into_inner();
@@ -217,36 +203,8 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
                     .unwrap();
 
                 ctx.start_transaction().await.unwrap();
-
-                let result = ctx.get_checkout(&path_params.id).await;
-
-                if let Err(err) = result {
-                    ctx.abort_transaction().await.unwrap();
-                    return Err(HttpError {
-                        status_code: StatusCode::BAD_REQUEST,
-                        error_code: Some(err.code.clone()),
-                        external_message: err.message.clone(),
-                        internal_message: err.message.clone(),
-                    });
-                }
-
-                let maybe_co = result.unwrap();
-
-                if maybe_co.is_none() {
-                    ctx.abort_transaction().await.unwrap();
-                    return Err(HttpError {
-                        status_code: StatusCode::NOT_FOUND,
-                        error_code: Some("NOT_FOUND".to_string()),
-                        external_message: "the requested checkout could not be found".to_string(),
-                        internal_message: "the requested checkout could not be found".to_string(),
-                    });
-                }
-
-                let mut co = maybe_co.unwrap();
-
-                match co.remove_item(&mut ctx, body_params.sku.clone(), body_params.quantity).await {
-                    Ok(_) => {
-                        ctx.set_checkout(&co.id, &co).await.unwrap();
+                match remove_item_handler(&mut ctx, &path_params.id, body_params.sku.clone(), body_params.quantity).await {
+                    Ok(co) => {
                         ctx.commit_transaction().await.unwrap();
                         Ok(HttpResponseOk(co))
                     }
@@ -266,9 +224,9 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
                 method = PATCH,
                 path = "/checkout/{id}/confirm_items",
             }]
-            pub async fn confirm_items_handler(
+            pub async fn confirm_items_endpoint(
                 req_ctx: Arc<RequestContext<ServerContext>>,
-                path: Path<StandardPathParams>,
+                path: Path<IdParam>,
             ) -> Result<HttpResponseOk<Checkout>, HttpError> {
                 let path_params = path.into_inner();
                 let mut ctx = req_ctx
@@ -280,35 +238,8 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
 
                 ctx.start_transaction().await.unwrap();
 
-                let result = ctx.get_checkout(&path_params.id).await;
-
-                if let Err(err) = result {
-                    ctx.abort_transaction().await.unwrap();
-                    return Err(HttpError {
-                        status_code: StatusCode::BAD_REQUEST,
-                        error_code: Some(err.code.clone()),
-                        external_message: err.message.clone(),
-                        internal_message: err.message.clone(),
-                    });
-                }
-
-                let maybe_co = result.unwrap();
-
-                if maybe_co.is_none() {
-                    ctx.abort_transaction().await.unwrap();
-                    return Err(HttpError {
-                        status_code: StatusCode::NOT_FOUND,
-                        error_code: Some("NOT_FOUND".to_string()),
-                        external_message: "the requested checkout could not be found".to_string(),
-                        internal_message: "the requested checkout could not be found".to_string(),
-                    });
-                }
-
-                let mut co = maybe_co.unwrap();
-
-                match co.confirm_items(&mut ctx).await {
-                    Ok(_) => {
-                        ctx.set_checkout(&co.id, &co).await.unwrap();
+                match confirm_items_handler(&mut ctx, &path_params.id).await {
+                    Ok(co) => {
                         ctx.commit_transaction().await.unwrap();
                         Ok(HttpResponseOk(co))
                     }
@@ -328,9 +259,9 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
                 method = PATCH,
                 path = "/checkout/{id}/update_contact",
             }]
-            pub async fn update_contact_handler(
+            pub async fn update_contact_endpoint(
                 req_ctx: Arc<RequestContext<ServerContext>>,
-                path: Path<StandardPathParams>,
+                path: Path<IdParam>,
                 body: TypedBody<Contact>,
             ) -> Result<HttpResponseOk<Checkout>, HttpError> {
                 let path_params = path.into_inner();
@@ -343,36 +274,8 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
                     .unwrap();
 
                 ctx.start_transaction().await.unwrap();
-
-                let result = ctx.get_checkout(&path_params.id).await;
-
-                if let Err(err) = result {
-                    ctx.abort_transaction().await.unwrap();
-                    return Err(HttpError {
-                        status_code: StatusCode::BAD_REQUEST,
-                        error_code: Some(err.code.clone()),
-                        external_message: err.message.clone(),
-                        internal_message: err.message.clone(),
-                    });
-                }
-
-                let maybe_co = result.unwrap();
-
-                if maybe_co.is_none() {
-                    ctx.abort_transaction().await.unwrap();
-                    return Err(HttpError {
-                        status_code: StatusCode::NOT_FOUND,
-                        error_code: Some("NOT_FOUND".to_string()),
-                        external_message: "the requested checkout could not be found".to_string(),
-                        internal_message: "the requested checkout could not be found".to_string(),
-                    });
-                }
-
-                let mut co = maybe_co.unwrap();
-
-                match co.update_contact(&mut ctx, contact).await {
-                    Ok(_) => {
-                        ctx.set_checkout(&co.id, &co).await.unwrap();
+                match update_contact_handler(&mut ctx, &path_params.id, contact).await {
+                    Ok(co) => {
                         ctx.commit_transaction().await.unwrap();
                         Ok(HttpResponseOk(co))
                     }
@@ -392,9 +295,9 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
                 method = PATCH,
                 path = "/checkout/{id}/update_shipping_address",
             }]
-            pub async fn update_shipping_address_handler(
+            pub async fn update_shipping_address_endpoint(
                 req_ctx: Arc<RequestContext<ServerContext>>,
-                path: Path<StandardPathParams>,
+                path: Path<IdParam>,
                 body: TypedBody<Address>,
             ) -> Result<HttpResponseOk<Checkout>, HttpError> {
                 let path_params = path.into_inner();
@@ -407,36 +310,8 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
                     .unwrap();
 
                 ctx.start_transaction().await.unwrap();
-
-                let result = ctx.get_checkout(&path_params.id).await;
-
-                if let Err(err) = result {
-                    ctx.abort_transaction().await.unwrap();
-                    return Err(HttpError {
-                        status_code: StatusCode::BAD_REQUEST,
-                        error_code: Some(err.code.clone()),
-                        external_message: err.message.clone(),
-                        internal_message: err.message.clone(),
-                    });
-                }
-
-                let maybe_co = result.unwrap();
-
-                if maybe_co.is_none() {
-                    ctx.abort_transaction().await.unwrap();
-                    return Err(HttpError {
-                        status_code: StatusCode::NOT_FOUND,
-                        error_code: Some("NOT_FOUND".to_string()),
-                        external_message: "the requested checkout could not be found".to_string(),
-                        internal_message: "the requested checkout could not be found".to_string(),
-                    });
-                }
-
-                let mut co = maybe_co.unwrap();
-
-                match co.update_shipping_address(&mut ctx, shipping_address).await {
-                    Ok(_) => {
-                        ctx.set_checkout(&co.id, &co).await.unwrap();
+                match update_shipping_address_handler(&mut ctx, &path_params.id, shipping_address).await {
+                    Ok(co) => {
                         ctx.commit_transaction().await.unwrap();
                         Ok(HttpResponseOk(co))
                     }
@@ -456,9 +331,9 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
                 method = PATCH,
                 path = "/checkout/{id}/update_fulfillment_type",
             }]
-            pub async fn update_fulfillment_type_handler(
+            pub async fn update_fulfillment_type_endpoint(
                 req_ctx: Arc<RequestContext<ServerContext>>,
-                path: Path<StandardPathParams>,
+                path: Path<IdParam>,
                 body: TypedBody<FulfillmentTypeSelection>,
             ) -> Result<HttpResponseOk<Checkout>, HttpError> {
                 let path_params = path.into_inner();
@@ -471,36 +346,44 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
                     .unwrap();
 
                 ctx.start_transaction().await.unwrap();
-
-                let result = ctx.get_checkout(&path_params.id).await;
-
-                if let Err(err) = result {
-                    ctx.abort_transaction().await.unwrap();
-                    return Err(HttpError {
-                        status_code: StatusCode::BAD_REQUEST,
-                        error_code: Some(err.code.clone()),
-                        external_message: err.message.clone(),
-                        internal_message: err.message.clone(),
-                    });
+                match update_fulfillment_type_handler(&mut ctx, &path_params.id, fulfillment_type).await {
+                    Ok(co) => {
+                        ctx.commit_transaction().await.unwrap();
+                        Ok(HttpResponseOk(co))
+                    }
+                    Err(err) => {
+                        ctx.abort_transaction().await.unwrap();
+                        Err(HttpError {
+                            status_code: StatusCode::BAD_REQUEST,
+                            error_code: Some(err.code.clone()),
+                            external_message: err.message.clone(),
+                            internal_message: err.message.clone(),
+                        })
+                    }
                 }
+            }
 
-                let maybe_co = result.unwrap();
+            #[endpoint {
+                method = PATCH,
+                path = "/checkout/{id}/initiate_payment",
+            }]
+            pub async fn initiate_payment_endpoint(
+                req_ctx: Arc<RequestContext<ServerContext>>,
+                path: Path<IdParam>,
+                body: TypedBody<<super::context_type_name as PaymentProcessor>::InitiateArgs>,
+            ) -> Result<HttpResponseOk<Checkout>, HttpError> {
+                let path_params = path.into_inner();
+                let args = body.into_inner();
+                let mut ctx = req_ctx
+                    .context()
+                    .context_provider
+                    .new_context()
+                    .await
+                    .unwrap();
 
-                if maybe_co.is_none() {
-                    ctx.abort_transaction().await.unwrap();
-                    return Err(HttpError {
-                        status_code: StatusCode::NOT_FOUND,
-                        error_code: Some("NOT_FOUND".to_string()),
-                        external_message: "the requested checkout could not be found".to_string(),
-                        internal_message: "the requested checkout could not be found".to_string(),
-                    });
-                }
-
-                let mut co = maybe_co.unwrap();
-
-                match co.update_fulfillment_type(&mut ctx, fulfillment_type).await {
-                    Ok(_) => {
-                        ctx.set_checkout(&co.id, &co).await.unwrap();
+                ctx.start_transaction().await.unwrap();
+                match initiate_payment_handler(&mut ctx, &path_params.id, args).await {
+                    Ok(co) => {
                         ctx.commit_transaction().await.unwrap();
                         Ok(HttpResponseOk(co))
                     }
@@ -522,14 +405,14 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
 
             fn api_description() -> checkout_server::ApiDescription<checkout_server::ServerContext> {
                 let mut api = checkout_server::ApiDescription::<checkout_server::ServerContext>::new();
-                api.register(checkout_server::create_handler).unwrap();
-                api.register(checkout_server::add_item_handler).unwrap();
-                api.register(checkout_server::remove_item_handler).unwrap();
-                api.register(checkout_server::confirm_items_handler).unwrap();
-                api.register(checkout_server::update_contact_handler).unwrap();
-                api.register(checkout_server::update_shipping_address_handler).unwrap();
-                api.register(checkout_server::update_fulfillment_type_handler).unwrap();
-                api
+                api.register(checkout_server::create_endpoint).unwrap();
+                api.register(checkout_server::add_item_endpoint).unwrap();
+                api.register(checkout_server::remove_item_endpoint).unwrap();
+                api.register(checkout_server::confirm_items_endpoint).unwrap();
+                api.register(checkout_server::update_contact_endpoint).unwrap();
+                api.register(checkout_server::update_shipping_address_endpoint).unwrap();
+                api.register(checkout_server::update_fulfillment_type_endpoint).unwrap();
+                api.register(checkout_server::initiate_payment_endpoint).unwrap();
             }
 
             fn new_server(
@@ -550,6 +433,13 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
 
                 let api = Self::api_description();
 
+                let config = checkout_server::get_config();
+
+                let internal_context = checkout_server::InternalContext::new(config.google_application_credentials.clone()).map_err(|_| checkout_server::Error {
+                    code: "500".to_string(),
+                    message: "failed to create internal context".to_string(),
+                })?;
+
                 checkout_server::HttpServerStarter::new(
                     &checkout_server::ConfigDropshot {
                         bind_address: bind_address.parse().unwrap(),
@@ -557,6 +447,7 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
                     },
                     api,
                     checkout_server::ServerContext {
+                        internal_context,
                         context_provider: self,
                     },
                     &log,
